#!/bin/sh

usage() {
	PROGNAME=`basename $0`
	echo
	echo -e "\033[1m$PROGNAME\033[0m"
	echo "Usage: $PROGNAME [-d|-e] [-c] ARG1 ARG2 ARG3"
	echo -e "\tDefault action is to produce a delta or apply patch"
	echo
	echo -e "\t-d\tApply patch ARG1 using file ARG2 producing ARG3"
	echo -e "\t-e\tProduce a delta from ARG1 to ARG2 producing ARG3"
	echo -e "\t-c\tCopy mode"
}

if [ $# -ne 4 ] && [ $# -ne 5 ] ; then
	usage
    exit 1;
fi

decode() {
	DIFF=$1
	if [ "$DIFF" = "bin" ]
	then
		xdelta3 -fds "$SOURCE" "$DELTA" "$DEST" 2>&1
	else
		cp -f "$DELTA" "$DEST"
	fi
}

encode() {
	DIFF=$1
	if [ "$DIFF" = "bin" ]
	then
		xdelta3 -e -9 -S djw -fs "$SOURCE" "$DEST" "$DELTA"
	else
		cp -f "$DEST" "$DELTA"
	fi
}

ACTION=
DIFF="bin"
while getopts dec option
do	
	case "$option" in
	d)	ACTION="patch"
		;;
	e)	ACTION="delta"
		;;
	c)	DIFF="copy"
		;;
	esac
done
shift `expr $OPTIND - 1`

SOURCE=$1
[ ! -f $SOURCE ] && {
	echo "Error: SOURCE file '$SOURCE' does not exist"
	exit 1
}

if [ "$ACTION" = "patch" ] ; then
	DELTA=$2
	DEST=$3
	[ ! -f $DELTA ] && {
	echo "Error: DELTA file '$DELTA' does not exist"
		exit 1
	}
	decode $DIFF
else
if [ "$ACTION" = "delta" ] ; then
	DEST=$2
	DELTA=$3
	[ ! -f $DEST ] && {
		echo "Error: DEST file '$DEST' does not exist"
		exit 1
	}
	encode $DIFF
else
	echo "Error: action '$ACTION' is unknown"
fi
fi

