#!/bin/sh
#set -x

usage() {
	PROGNAME=`basename $0`
	echo
	echo -e "\033[1m$PROGNAME\033[0m"
	echo "Usage: $PROGNAME [-c flaglist] [-l filelist] [-x xfilelist] payloaddir destinputfile"
	echo -e "\tDefault action is to apply zdelta patch payload, limiting to those in filelist or excluding those in xfilelist"
	echo
	echo -e "\t-c\tcondition flag(s) to be set"
	echo -e "\t-l\trestrict patching to files in filelist"
	echo -e "\t-x\texclude files in xfilelist from patching"
}

if [ $# -ne 2 -a $# -ne 4 -a $# -ne 6 -a $# -ne 8 ] ; then
	usage
    exit 1;
fi


CONDITION_LIST=
RESTRICT_LIST=
EXCLUDE_LIST=

while getopts c:l:x: option
do	
	case "$option" in
	c)	CONDITION_LIST=$OPTARG
		;;
	l)	RESTRICT_LIST=$OPTARG
		;;
	x)	EXCLUDE_LIST=$OPTARG
		;;
	esac
done
shift `expr $OPTIND - 1`

PAYLOAD_DIR=$1
if [ ! -d $PAYLOAD_DIR ] ; then
	echo "Directory '$PAYLOAD_DIR' does not exist"
	exit 1
fi

PATCH_DESTINATION_FILE=$2
if [ ! -f $PATCH_DESTINATION_FILE ] ; then
	echo "File '$PATCH_DESTINATION_FILE' does not exist"
	exit 1
fi


###############
## FUNCTIONS ##
###############
. zdpathlib.sh

##########
## MAIN ##
##########

while read LINE
do
	INDEX=`echo "$LINE" | cut -f 1 | sed -e "s/ //g"` ; # echo INDEX=$INDEX
	
	# do nothing for title line
	if [ $INDEX -eq 0 ] ; then
	    continue;
	fi
	
	# DestInput.txt has the following colums
	#  1 0
	#  2 To Version
	#  3 File Name
	#  4 Folder Path on Appliance (Destination)
	#  5 Condition
	#  6 Owner (Only used for new files)
	#  7 Permissions (Only used for new files)
	#  8 New Directory (Only used for new files)
	#  9 Directory Owner (Only used for new files)
	# 10 Directory Permissions (Only used for new files)
	TOVER=`echo "$LINE" | cut -f 2 | sed -e "s/ //g"` ; # echo TOVER=$TOVER
	FNAME=`echo "$LINE" | cut -f 3 | sed -e "s/ //g"` ; # echo FNAME=$FNAME
	FPATH=`echo "$LINE" | cut -f 4 | sed -e "s/ //g"` ; # echo FPATH=$FPATH
	CONDS=`echo "$LINE" | cut -f 5 | sed -e "s/ //g"` ; # echo CONDS=$CONDS
	OWNER=`echo "$LINE" | cut -f 6 | sed -e "s/ //g"` ; # echo OWNER=$OWNER
	PERMS=`echo "$LINE" | cut -f 7 | sed -e "s/ //g"` ; # echo PERMS=$PERMS
	NEWDR=`echo "$LINE" | cut -f 8 | sed -e "s/ //g"` ; # echo NEWDR=$PERMS
	DIROW=`echo "$LINE" | cut -f 9 | sed -e "s/ //g"` ; # echo PERMS=$DIRPM
	DIRPM=`echo "$LINE" | cut -f 10 | sed -e "s/ //g"` ; # echo PERMS=$DIRPM
	
	# delete file on appliance if destination version is null
	if [ "$TOVER" == "0" ]; then
		log "Delete file $FNAME\n"
		rm -rf "$FPATH/$FNAME"
		continue
	fi
	
	# restrict input
	[ -n "$RESTRICT_LIST" ] && {
		found=
		for rfile in `echo $RESTRICT_LIST | tr "," " "`;
		do
			if [ "$rfile" = "$FNAME" ] ; then
				found=1
				break
			fi
		done
		[ -z "$found" ] && {
			log "Excluding file $FNAME\n"
			continue
		}
	}
	
	# exclude input
	[ -n "$EXCLUDE_LIST" ] && {
		found=
		for xfile in `echo $EXCLUDE_LIST | tr "," " "`;
		do
			if [ "$xfile" = "$FNAME" ] ; then
				found=1
				break
			fi
		done
		[ -n "$found" ] && {
			log "Excluding file $FNAME\n"
			continue
		}
	}
	
		
	# File is patched only if condition is fulfilled
	check_condition ${CONDS} && {
		if [ -f ${PAYLOAD_DIR}/${INDEX} ]; then
			patch_existing_file ${PAYLOAD_DIR}/${INDEX} $FNAME $FPATH
		else
		if [ -f ${PAYLOAD_DIR}/${INDEX}.new.zip ]; then
			install_new_file ${PAYLOAD_DIR}/${INDEX}.new.zip $FNAME $FPATH $CONDS $OWNER $PERMS $NEWDR $DIROW $DIRPM
		else
		if [ -f ${PAYLOAD_DIR}/${INDEX}.zip ]; then
			zdr ${PAYLOAD_DIR}/${INDEX}.zip $FNAME $FPATH $CONDS $OWNER $PERMS $NEWDR $DIROW $DIRPM
		else
			log "[ERROR] No file found for INDEX ($INDEX).\n"
			continue
		fi
		fi
		fi
		# set file ownership
		[ -n "${OWNER}" ] && {
			chown ${OWNER} $FPATH/$FNAME
		}
		# set file permissions
		[ -n "${PERMS}" ] && {
			chmod ${PERMS} $FPATH/$FNAME
		}
	}
done < $PATCH_DESTINATION_FILE
