#!/bin/sh
#set -x

###############
## FUNCTIONS ##
###############

#-----------------------------------
#	log
#-----------------------------------
log()
{
	printf "`date "+%Y-%m-%d %H:%M:%S"` $*"
}


#-----------------------------------
#	check_condition
#-----------------------------------
check_condition()
{
    CONDITIONS2TEST=$1
	
	[ -z "$CONDITIONS2TEST" ] && {
		return 0
	}

	# prefefine condition NONE
    echo $CONDITIONS2TEST | grep -q "NONE" && {
		return 0
    }
	
    for cdn2test in `echo $CONDITIONS2TEST | sed -e 's/|/ /g'`;
	do
		for cdn in `echo $CONDITION_LIST | sed -e 's/,/ /g'`;
		do
			[ ${cdn2test} = ${cdn} ] && {
				return 0
			}
		done	
    done
	log "Condition $CONDITIONS2TEST is not fulfilled\n"
    return 1
}


#-----------------------------------
#	patch_existing_file
#-----------------------------------
patch_existing_file()
{
    PYLD=$1
    FILE=$2
    DEST=$3

	log "patch_existing_file started with ${PYLD} ${FILE} ${DEST} ${CDTN} \n"
	if [ -f ${DEST}/${FILE} ]; then 
	    log "Calling zdbin.sh -d ${ZDBINOPT} ${DEST}/${FILE} ${PYLD} ${DEST}/${FILE}.new ...\n"
	    /bin/rm -f ${DEST}/${FILE}.new
            zdbin.sh -d ${ZDBINOPT} ${DEST}/${FILE} ${PYLD} ${DEST}/${FILE}.new
	    if [ $? != 0 ]
	    then
	        log "[ERROR] Decoding failed"
	    else
	       /bin/cp -f ${DEST}/${FILE}.new ${DEST}/${FILE}
	       /bin/rm -f ${DEST}/${FILE}.new
	    fi
	else 
	    log "[ERROR] File to patch does not exist: ${DEST}/${FILE}\n"
	fi
}

#-----------------------------------
#	install_new_file
#-----------------------------------
install_new_file()
{
    PYLD=$1
    FILE=$2
    DEST=$3
    CDTN=$4
    FILO=$5
    FILP=$6
    NEWD=$7
    DIRO=$8
    DIRP=$9

    check_condition ${CDTN} && {
		log "install_new_file started ${PYLD} ${FILE} ${DEST} ${CDTN} ${FILO} ${FILP} \n"

		if [ "$NEWD" != "" ]; then
			log "Creatng directory ${DEST}...\n"
			mkdir -p ${DEST}
			chown -R ${DIRO} ${NEWD}
			chmod -R ${DIRP} ${NEWD}
		fi    
		if [ -f ${DEST}/${FILE} ]; then
			log "[WARN] Overwriting an existing file: ${DEST}/${FILE}\n"
			/bin/mv -f ${DEST}/${FILE} ${DEST}/${FILE}_bak
		fi
		unzip -d ${DEST} ${PYLD} 
		chown ${FILO} ${DEST}/${FILE}
		chmod ${FILP} ${DEST}/${FILE}
		if [ -f ${DEST}/${FILE} ]; then
			/bin/rm -f ${DEST}/${FILE}_bak
		else
			log "[ERROR] Installation failed and restoring original file: ${DEST}/${FILE}\n"
			/bin/mv -f ${DEST}/${FILE}_bak ${DEST}/${FILE}
		fi
    }
}

#-----------------------------------
#	zdae
#-----------------------------------
zdae() # add entry
{
    F2P="$1" ; DELTA="$2" ; E2P="$3" # F2P = file to patch, E2P = entry to patch (in F2P)
    DIRNAME=`dirname $E2P`
    TMPDIR=`mktemp -d`
	(
		cd $TMPDIR; 
		if [[ $E2P == */ ]]; then
			mkdir -p $E2P;
		else 
			mkdir -p $DIRNAME; cp $DELTA $E2P; touch $E2P; 
		fi
		zip -u "$F2P" $E2P
	)
    rm -rf $TMPDIR
}

#-----------------------------------
#	zdpe
#-----------------------------------
zdpe() # patch entry
{
    F2P="$1" ; DELTA="$2" ; E2P="$3" # F2P = file to patch, E2P = entry to patch (in F2P)
    TMPDIR=`mktemp -d`
	(cd $TMPDIR ; unzip "$F2P" "$E2P")
	zdbin.sh -d ${ZDBINOPT} "$TMPDIR/$E2P" "$DELTA" "$TMPDIR/$E2P" # patch in place
    (cd $TMPDIR ; zip "$F2P" "$E2P")  
    rm -rf $TMPDIR
}

#-----------------------------------
#	zdde
#-----------------------------------
zdde() # delete entry
{
    DEST="$1" ; ENTRY="$2" # ENTRY = entry to delete in DEST, DEST = zip file to modify
    zipinfo -1 "$DEST" | fgrep "$ENTRY" &> /dev/null
    HASENTRY=$?
    # Delete entry if it exists (avoids zip error) 
    if [ $HASENTRY = 0 ]
    then
        zip -d "$DEST" "$ENTRY"
    fi
}

#-----------------------------------
#	zdme
#-----------------------------------
zdme() # move entry
{
    F2P="$1" ; E2M="$2" ; DEST="$3" # F2P = file to patch, E2M = entry to move (in F2P)
	DIRNAME=`dirname $DEST`
    TMPDIR=`mktemp -d`
	(cd $TMPDIR ; mkdir -p $DIRNAME; unzip "$F2P" "$E2M" ; mv "$E2M" "$DEST" ; zip -u "$F2P" $DEST ; zip -d "$F2P" "$E2M" ; )
	rm -rf $TMPDIR
}

#-----------------------------------
#	zdce
#-----------------------------------
zdce() # copy entry
{
    F2P="$1" ; E2C="$2" ; DEST="$3" # F2P = file to patch, E2C = entry to copy (in F2P)
	DIRNAME=`dirname $DEST`
    TMPDIR=`mktemp -d`
	(cd $TMPDIR ; mkdir -p $DIRNAME; unzip "$F2P" "$E2C" ; cp -r "$TMPDIR/$E2C" "$TMPDIR/$DEST" ; zip -u "$F2P" $DEST )
	rm -rf $TMPDIR
}

#-----------------------------------
#	zdr
#-----------------------------------
zdr() {

	PYLD=$1 # zipped payload file, ex: 1.zip, 5-9.zip
    FILE=$2
    DEST=$3
    CDTN=$4
    FILO=$5
    FILP=$6
    NEWD=$7
    DIRO=$8
    DIRP=$9
		
	# file to patch
	local FILE2PATCH=$DEST/$FILE
	
	# unzip payload to temp folder
	PDIR=`mktemp -d` ; unzip $PYLD -d $PDIR
		
	PFILES=$PDIR/*
	for f in $PFILES
	do
		fname=`basename $f` 
		
		# i-j.zip
		#--------
		regex="^[0-9]+-[0-9]+.zip$"
		if [[ $fname =~ $regex ]]; then
			(
				j=`echo ${fname%.zip} | cut -d- -f2` #index in dest.bom
				PATHDR=`cat "$PDIR/dest.bom" | head -n$j | tail -n1 | cut -d' ' -f2`
				PDIR_R=`mktemp -d`
				(cd $PDIR_R ; unzip "$FILE2PATCH" "$PATHDR")
				NAMED=`basename $PATHDR`
				DIRD=`dirname $PATHDR`
				zdr $f $NAMED $PDIR_R/$DIRD $CDTN $FILO $FILP $NEWD $DIRO $DIRP
				(cd $PDIR_R ; zip $FILE2PATCH $PATHDR )
				rm -rf $PDIR_R
			)
		fi
				
		# i.new
		#------
		# find PATHD at index i in dest.bom
		# zdae "$FILE2PATCH" f $PATHD
		regex="^[0-9]+\.new$"
		if [[ $fname =~ $regex ]]; then
			i=${fname%.*} # index in dest.bom
			PATHD=`cat "$PDIR/dest.bom" | head -n$i | tail -n1 | cut -d' ' -f2` # path in dest.bom
			zdae $FILE2PATCH $f $PATHD
		fi
		
		# i-j
		#----
		# find PATHD at index j in dest.bom
		regex="^[0-9]+-[0-9]+$"
		if [[ $fname =~ $regex ]]; then
			j=`echo $fname | cut -d- -f2` #index in dest.bom
			PATHD=`cat "$PDIR/dest.bom" | head -n$j | tail -n1 | cut -d' ' -f2`
			zdpe "$FILE2PATCH" $f $PATHD
		fi
				
	done
	
	# Copy/Move
	#----------
	cat "$PDIR/dest.bom" | while read HASH FILEPATH
	do
		# no change, continue
		[ -n "`grep \"^$HASH $FILEPATH$\" $PDIR/source.bom`" ] && {
			continue
		}

		# same hash exists in source
		PATHS=`grep -m 1 "^$HASH " "$PDIR/source.bom" | cut -d' ' -f2`
		[ -n "$PATHS" ] && {
			CNTHD=`grep -c "^$HASH " $PDIR/dest.bom`
			CNTHS=`grep -c "^$HASH " $PDIR/source.bom`
			if [ "$CNTHD" = "1" -a "$CNTHS" = "1" ] ; then
				# move existing file to new path
				zdme "$FILE2PATCH" $PATHS $FILEPATH
			else
				# copy existing file to new path
				zdce "$FILE2PATCH" $PATHS $FILEPATH
			fi
		}
	done
	
	# Remove
	#-------
	cat "$PDIR/source.bom" | while read HASH FILEPATH
	do
		# filename in source but not in destination => delete it (except if it's a moved file)
		[ -z "`grep \" $FILEPATH$\" $PDIR/dest.bom`" ] && {
			CNTHD=`grep -c "^$HASH " $PDIR/dest.bom`
			CNTHS=`grep -c "^$HASH " $PDIR/source.bom`
			if [ "$CNTHD" != "1" -o "$CNTHS" != "1" ] ; then
				zdde "$FILE2PATCH" "$FILEPATH"
			fi
		}
	done
	
	rm -rf $PDIR
}
