#!/bin/sh
#set -x

PROGNAME=`basename $0`

if [ $# -ne 3 ]; then
	echo "$PROGNAME is a command line tool that generates software update payload."
	echo "usage: $PROGNAME <resursion level> <input file> <output dir>"
	echo
	exit 1;
fi

# root folder of input
INPUTDIR=/work/integrate

# folder to store payload
PAYLOADDIR=$3
rm -rf $PAYLOADDIR ; mkdir $PAYLOADDIR


#  1 0
#  2 Module
#  3 From Version
#  4 To Version
#  5 File Name
#  6 Folder Path in Module Distribution View
#  7 File Size
#  8 Delta File
#  9 Delta Size
# 10 Patch
# 11 Add
# 12 Remove
# 13 Move
# 14 Copy
# 15 Total

COUNTERFILE=/tmp/counter


###############
## FUNCTIONS ##
###############

log()
{
	printf "`date "+%Y-%m-%d %H:%M:%S"` $LVL $*" 1>&2
}

##########
## MAIN ##
##########

while read LINE
do
	# remove previous statistic file
	rm -f $COUNTERFILE

	NDX=`echo "$LINE" | cut -f 1` ; # echo NDX=$NDX
	if [ $NDX -ne 0 ] ; then
		MODNAME=`echo "$LINE" | cut -f 2` ; # echo MODNAME=$MODNAME
		FROMVER=`echo "$LINE" | cut -f 3` ; # echo FROMVER=$FROMVER
		TOVER=`echo "$LINE" | cut -f 4`   ; # echo TOVER=$TOVER
		FNAME=`echo "$LINE" | cut -f 5`   ; # echo FNAME=$FNAME
		FPATH=`echo "$LINE" | cut -f 6`   ; # echo FPATH=$FPATH
		
		TMP_DIR_S=/tmp/.$$_zdpayload_S
		TMP_DIR_D=/tmp/.$$_zdpayload_D
		mkdir $TMP_DIR_S
		mkdir $TMP_DIR_D
			
		IS_RPM=`echo $FPATH | grep "?"`
		if [ ".$IS_RPM" != "." ]
		then
			RPM_PATH=`echo $FPATH | cut -d? -f1`
			FILE_IN_RPM_PATH=`echo $FPATH | cut -d? -f2`

			# extract the file
			(cd $TMP_DIR_S;rpm2cpio $INPUTDIR/${MODNAME}_distrib\#$FROMVER/${MODNAME}_distrib/$RPM_PATH | cpio -ivd $FILE_IN_RPM_PATH/$FNAME)
			(cd $TMP_DIR_D;rpm2cpio $INPUTDIR/${MODNAME}_distrib\#$TOVER/${MODNAME}_distrib/$RPM_PATH | cpio -ivd $FILE_IN_RPM_PATH/$FNAME)
			
			SOURCE=$TMP_DIR_S/$FILE_IN_RPM_PATH/$FNAME
			DEST=$TMP_DIR_D/$FILE_IN_RPM_PATH/$FNAME			
		else
			SOURCE=$INPUTDIR/${MODNAME}_distrib\#$FROMVER/${MODNAME}_distrib/${FPATH}/${FNAME}
			DEST=$INPUTDIR/${MODNAME}_distrib\#$TOVER/${MODNAME}_distrib/${FPATH}/${FNAME}
		fi
		
		SIZESRC=`stat --format=%s "$SOURCE" 2>/dev/null`
		SIZEDST=`stat --format=%s "$DEST" 2>/dev/null`
		
		if [ "$FROMVER" != "0" ] ; then
			DELTAFILE=$PAYLOADDIR/$NDX
			zd.sh "$1" "$SOURCE" "$DEST" "$DELTAFILE" 1>&2
			if [ $? -eq 1 ] ; then DELTAFILE="${DELTAFILE}.zip" ; fi
		else
			DELTAFILE=$PAYLOADDIR/$NDX.new.zip
			log "NEW FILE ($NDX), ZIP IT..."
			zip -qj9 "$DELTAFILE" "$DEST"
			log "DONE\n"
		fi

		DELTASIZE=`stat --format=%s "$DELTAFILE" 2>/dev/null`
		
		COUNTPATCH=0
		COUNTADD=0
		COUNTREMOVE=0
		COUNTMOVE=0
		COUNTCOPY=0
		COUNTTOTAL=0
		if [ -f "$COUNTERFILE" ] ; then
			COUNTPATCH=`grep -m 1 "^COUNTPATCH=[0-9]*" "$COUNTERFILE" | cut -d= -f2`
			COUNTADD=`grep -m 1 "^COUNTADD=[0-9]*" "$COUNTERFILE" | cut -d= -f2`
			COUNTREMOVE=`grep -m 1 "^COUNTREMOVE=[0-9]*" "$COUNTERFILE" | cut -d= -f2`
			COUNTMOVE=`grep -m 1 "^COUNTMOVE=[0-9]*" "$COUNTERFILE" | cut -d= -f2`
			COUNTCOPY=`grep -m 1 "^COUNTCOPY=[0-9]*" "$COUNTERFILE" | cut -d= -f2`
			: ${COUNTPATCH:=0}
			: ${COUNTADD:=0}
			: ${COUNTREMOVE:=0}
			: ${COUNTMOVE:=0}
			: ${COUNTCOPY:=0}
			COUNTTOTAL=`expr $COUNTPATCH + $COUNTADD + $COUNTREMOVE + $COUNTMOVE + $COUNTCOPY`
		fi

		printf "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n"\
			"$NDX"\
			"$MODNAME"\
			"$FROMVER"\
			"$TOVER"\
			"$FNAME"\
			"$FPATH"\
			"$SIZESRC"\
			"$SIZEDST"\
			"`basename $DELTAFILE`"\
			"$DELTASIZE"\
			"$COUNTPATCH"\
			"$COUNTADD"\
			"$COUNTREMOVE"\
			"$COUNTMOVE"\
			"$COUNTCOPY"\
			"$COUNTTOTAL"
		
		# remove the TPM folder
		rm -rf $TMP_DIR_S
		rm -rf $TMP_DIR_D
	else
		echo "$LINE"
	fi
done < $2
