#!/bin/sh

###########################################################################################
#  (c) Copyright 2013 HID Global Corporation/ASSA ABLOY AB. All Rights Reserved.          #
#                                                                                         #
#  This program is an unpublished copyrighted work which is proprietary                   #
#  to HID Global Corporation/ASSA ABLOY AB. This computer program includes Confidential,  #
#  Proprietary Information and is a Trade Secret of HID Global Corporation/ASSA ABLOY AB. #
#  Any use, disclosure, modification and/or reproduction is prohibited                    #
#  unless authorized in writing by HID Global Corporation/ASSA ABLOY AB.                  #
#                                                                                         #
#  WARNING:  Unauthorized reproduction of this program as well as                         #
#  unauthorized preparation of derivative works based upon the                            #
#  program or distribution of copies by sale, rental, lease or                            #
#  lending are violations of federal copyright laws and state trade                       #
#  secret laws, punishable by civil and criminal penalties.                               #
###########################################################################################

UPDATER_HOME=/opt/hidglobal/ac.ac4linux
VERSION_RPM_NAME=unused # No version RPM for ActivClient
INSTALL_HISTORY_FILE=$UPDATER_HOME/installhistory.properties
LOCALE_FILE=$UPDATER_HOME/locale.properties
SOFTWARE_UPDATER_LOCALE_FILE=${LOCALE_FILE}_softwareupdater
SOFTWARE_UPDATER_LOGFILE=/var/log/activclient/softwareupdater.log
FULLINSTALLER_TYPE="Full Installer"
ADDON_TYPE="Add-On"
HOTFIX_TYPE="Hot Fix"
SERVICEPACK_TYPE="Service Pack"
PATCH_TYPE="Patch"

logSoftwareUpdater()
{
	if [ -z "$1" ]
	then
		echo " " >> $SOFTWARE_UPDATER_LOGFILE
	else
		echo "`date`: $1" >> $SOFTWARE_UPDATER_LOGFILE
	fi
}

assert() {
	if [ "$1" -ne "0" ]; then
		logSoftwareUpdater "$2 ($1)"
		echo "$2 ($1)" >&2
		return $RC
	fi
}

getLocalProp() {
	logSoftwareUpdater "getLocalProp(): <$1>"
	fgrep "$1=" ${LOCALE_FILE} | cut -d"=" -f2 2> /dev/null
}

getCustomProp() {
	logSoftwareUpdater "getCustomProp(): <$1> <$2>"
	PACKAGE_NAME=$1
	rpm --nosignature -q -p $PACKAGE_NAME --qf "%{Description}\n" | fgrep "$2" | cut -d":" -f2- 2> /dev/null
}

setInstallHistoryProp() {
	logSoftwareUpdater "setInstallHistoryProp(): <$1> <$2> <$3>"
	local NAME=$1
	local VALUE=$2
	local RPM_TYPE=$3
	[ -f $INSTALL_HISTORY_FILE ] || {
		touch $INSTALL_HISTORY_FILE
	}
	cp -p $INSTALL_HISTORY_FILE $INSTALL_HISTORY_FILE.2
	fgrep -v "$NAME" $INSTALL_HISTORY_FILE.2 > $INSTALL_HISTORY_FILE
	echo "$NAME=$VALUE" >> $INSTALL_HISTORY_FILE
	[ ".$RPM_TYPE" != "." ] && echo "$VALUE=$RPM_TYPE" >> $INSTALL_HISTORY_FILE
	/bin/rm -f $INSTALL_HISTORY_FILE.2
}

verifyTransport() {
	# Check if the dependencies are correctly installed
	logSoftwareUpdater "verifyTransport(): <$1> <$2>"
	PACKAGE_NAME=$1
	RPM_TYPE=$2

	rpm -U --test $PACKAGE_NAME > tmp_rpm_test.txt 2>&1
	ret=$?
	if [ $ret != 0 ]
	then
		logSoftwareUpdater "verifyTransport(): `cat tmp_rpm_test.txt`"
	fi
	/bin/rm -f tmp_rpm_test.txt
	
	# If this is a Service Pack, check if a conflict was registered
	if [ "$RPM_TYPE" = "$SERVICEPACK_TYPE" ]
	then
		NAME="SERVICE_PACK_CONFLICT"
		CONFLICTING=`getLocalProp $NAME`
		if [ -n "$CONFLICTING" ]
		then
			logSoftwareUpdater "verifyTransport(): Known conflict: $CONFLICTING"
			ret=1
		fi
	fi

	logSoftwareUpdater "verifyTransport(): return <$ret>"
	return $ret
}

unpackTransport() {
	logSoftwareUpdater "unpackTransport(): <$1> <$2>"
	TRANSPORT_FILE=$1
	DEST=$2
	[ -d $DEST ] || {
		logSoftwareUpdater "unpackTransport(): $DEST does not exist"
		return 1
	}
	(cd $DEST; rpm2cpio $TRANSPORT_FILE | cpio -idv 2> /dev/null)
	assert $? "failed to extract transport payload"
	return 0
}

filterPatches() {
	logSoftwareUpdater "filterPatches(): <$1>"
	DIR=$1
	for file in `ls $DIR/*.rpm | fgrep -v $VERSION_RPM_NAME`
	do
		# Delete the patch if it is already installed
		RPM_NAME=`rpm -qp --qf "%{NAME}-%{VERSION}-%{RELEASE}" $file`
		rpm -q $RPM_NAME 1> /dev/null
		if [ "$?" = 0 ]
		then
			logSoftwareUpdater "filterPatches(): $RPM_NAME already installed"
			/bin/rm -f $file
		else
			# Delete the patch if the corresponding setup is not installed 
			SETUP_NAME=`rpm -qp --qf "%{NAME}" $file`
			rpm -q $SETUP_NAME 1> /dev/null
			[ "$?" != 0 ] && {
				logSoftwareUpdater "filterPatches(): $SETUP_NAME not installed, skip $RPM_NAME"
				/bin/rm -f $file
			}
		fi
	done
}

#   Returns:
#	 0: success
#	 1: failed
checkInstallability()
{
	logSoftwareUpdater "checkInstallability(): <$1> <$2>"
	PAYLOAD_DIR=$1
	TRANSACTIONSETS=$2
	RC=0

	# For all transaction sets, check if RPM can be installed
	echo $TRANSACTIONSETS | sed "s/,/\n/g" | sed "s/:/ /g" | while read TS OP
	do
		logSoftwareUpdater "checkInstallability(): Verify transaction set <$TS> <$OP>"
		DIR=$PAYLOAD_DIR/$TS

		case $OP in
			i) RPMOPT="-ivh" ;;
			u) RPMOPT="-Uvh" ;;
			f) RPMOPT="-Fvh" ;;
			p) RPMOPT="-ivh" ;;
		esac

		# Test install rpms if any (if transactionset is '.', ignore version RPM)
		ls $PAYLOAD_DIR/$TS/*.rpm 2> /dev/null | fgrep -v $VERSION_RPM_NAME 1> /dev/null
		if [ $? -eq 0 ]
		then
			rpm $RPMOPT --test `ls $PAYLOAD_DIR/$TS/*.rpm | fgrep -v $VERSION_RPM_NAME` > tmp_rpm_test.txt 2>&1
			ret=$?
			if [ ! $ret -eq 0 ]
			then
				logSoftwareUpdater "checkInstallability(): `cat tmp_rpm_test.txt`"
				/bin/rm -f tmp_rpm_test.txt

				logSoftwareUpdater "checkInstallability(): Installation of <$TS> <$OP> failed"
				RC=1
			fi
		fi
	done

	# Test install of the version RPM (if any: only HF and SP have one)
	ls $PAYLOAD_DIR/*.rpm 2> /dev/null | fgrep $VERSION_RPM_NAME 1> /dev/null
	[ $? -eq 0 ] && {
		rpm -U --test `ls $PAYLOAD_DIR/*.rpm | fgrep $VERSION_RPM_NAME` > tmp_rpm_test.txt 2>&1
		ret=$?
		if [ ! $ret -eq 0 ]
		then
			logSoftwareUpdater "checkInstallability(): `cat tmp_rpm_test.txt`"
			/bin/rm -f tmp_rpm_test.txt

			logSoftwareUpdater "checkInstallability(): Installation of version RPM failed"
			RC=1
		fi
	}

	return $RC
}

#
# installThePayload
#   PACKAGE_NAME is the name of the transport RPM
#   TRANSACTIONSETS must not be empty:
#	 either it is the value set in the transport RPM
#	 or the default value in the form ".:<option>'
#	 where option is i,u,f,p depending on the package type
#   Returns:
#	 0: success
#	 1: failed
#
installThePayload() {
	logSoftwareUpdater "installThePayload(): <$1> <$2> <$3> <$4>"
	PACKAGE_NAME=$1
	PAYLOAD_DIR=$2
	TRANSACTIONSETS=$3
	RPM_TYPE=$4
	
	echo $TRANSACTIONSETS | sed "s/,/\n/g" | sed "s/:/ /g" | while read TS OP
	do
		logSoftwareUpdater "installThePayload(): Verify/Install transaction set $TS ($OP)"
		case $OP in
			i) RPMOPT="-ivh" ;;
			u) RPMOPT="-Uvh" ;;
			f) RPMOPT="-Fvh" ;;
			p) RPMOPT="-ivh" ;;
		esac
		# Install rpms if any (if transactionset is '.', ignore version RPM)
		ls $PAYLOAD_DIR/$TS/*.rpm 2> /dev/null | fgrep -v $VERSION_RPM_NAME 1> /dev/null
		if [ $? -eq 0 ]
		then
			rpm $RPMOPT `ls $PAYLOAD_DIR/$TS/*.rpm | fgrep -v $VERSION_RPM_NAME`
			if [ ! $? -eq 0 ]
			then
				logSoftwareUpdater "installThePayload(): Installation of $TS ($OP) failed"
				return 1
			fi

			# Transaction set is now installed, store the mapping between
			# the transaction id and the package name
			#   we need one of the RPM in the transaction
			INNERPKG=`ls -1 $PAYLOAD_DIR/$TS/*.rpm | fgrep -v $VERSION_RPM_NAME | while read P ; do echo $P ; break ; done`
			INNERPKG_NAME=`rpm -qp --qf "%{NAME}-%{VERSION}-%{RELEASE}" $INNERPKG`
			#   we get the transaction id from the corresponding installed package
			TID=`rpm -q --qf %{INSTALLTID} $INNERPKG_NAME`
			# set RPM type in installhistory.properties
			logSoftwareUpdater "installThePayload(): Register transaction set $TID=$PACKAGE_NAME"
			setInstallHistoryProp $TID $PACKAGE_NAME "$RPM_TYPE"
		fi
		
	done
	RC=$? ; if [ ! $RC -eq 0 ] ; then return $RC ; fi

	# Install version RPM (if any: only HF and SP have one)
	ls $PAYLOAD_DIR/*.rpm 2> /dev/null | fgrep $VERSION_RPM_NAME 1> /dev/null
	[ $? -eq 0 ] && {
		rpm -U `ls $PAYLOAD_DIR/*.rpm | fgrep $VERSION_RPM_NAME`
		if [ ! $? -eq 0 ]
		then
			logSoftwareUpdater "installThePayload(): Installation of version RPM failed"
			return 1
		fi
	}
	
	return 0
}

verifyPayload() {
	logSoftwareUpdater "verifyPayload(): <$1> <$2>"
	PAYLOAD_DIR=$1
	TRANSACTIONSETS=$2

	echo $TRANSACTIONSETS | sed "s/,/\n/g" | sed "s/:/ /g" | while read TS OP
	do
		logSoftwareUpdater "verifyPayload(): Verify transaction set <$TS> <$OP>"
		if [ "$OP" = "p" ]
		then
			filterPatches $PAYLOAD_DIR/$TS
		fi
	done

	checkInstallability $PAYLOAD_DIR $TRANSACTIONSETS
	if [ ! $? -eq 0 ]
	then
		logSoftwareUpdater "verifyPayload(): verify RPMs install failed"
		return 1
	fi

	return 0
}

installPayload() {
	logSoftwareUpdater "installPayload(): <$1> <$2> <$3> <$4>"

	PACKAGE_NAME=$1
	PAYLOAD_DIR=$2
	TRANSACTIONSETS=$3
	RPM_TYPE=$4

	installThePayload $PACKAGE_NAME $PAYLOAD_DIR $TRANSACTIONSETS "$RPM_TYPE"
	RC=$?
	if [ "$RC" -eq "0" ]
	then
		logSoftwareUpdater "installPayload(): Installation of $PACKAGE_NAME succeeded"
	fi
	logSoftwareUpdater "installPayload(): Installation finished ($RC)"
	return $RC
}

updateConflictProp() {
	logSoftwareUpdater "updateConflictProp(): <$1> <$2>"
	
	PACKAGE_NAME=$1
	SET_SP_CONFLICT=$2
	
	# Work on a temporary file
	touch $LOCALE_FILE
	cp $LOCALE_FILE ${SOFTWARE_UPDATER_LOCALE_FILE}
	
	# if the transport RPM has servicepackconflict set,
	# register it in appliance_local.properties
	if [ "$SET_SP_CONFLICT" = "true" ]
	then
		# look whether it is already defined
		NAME="SERVICE_PACK_CONFLICT"
		CONFLICTING=`getLocalProp $NAME`
		logSoftwareUpdater "updateConflictProp(): NAME=<$NAME> , CONFLICTING=<$CONFLICTING>"

		# update property file
		cp -p ${SOFTWARE_UPDATER_LOCALE_FILE} ${SOFTWARE_UPDATER_LOCALE_FILE}.2
		fgrep -v "$NAME" ${SOFTWARE_UPDATER_LOCALE_FILE}.2 > ${SOFTWARE_UPDATER_LOCALE_FILE}
		if [ -z "$CONFLICTING" ]
		then
			logSoftwareUpdater "updateConflictProp(): adding <$NAME=$PACKAGE_NAME> in file ${SOFTWARE_UPDATER_LOCALE_FILE}"
			echo "$NAME=$PACKAGE_NAME" >> ${SOFTWARE_UPDATER_LOCALE_FILE}
		else
			logSoftwareUpdater "updateConflictProp(): adding <$NAME=$PACKAGE_NAME,$CONFLICTING> in file ${SOFTWARE_UPDATER_LOCALE_FILE}"
			echo "$NAME=$PACKAGE_NAME,$CONFLICTING" >> ${SOFTWARE_UPDATER_LOCALE_FILE}
		fi
		/bin/rm -f ${SOFTWARE_UPDATER_LOCALE_FILE}.2
	fi
	if [ "$SET_SP_CONFLICT" = "false" ]
	then
		# remove the "SERVICE_PACK_CONFLICT" property
		cp -p ${SOFTWARE_UPDATER_LOCALE_FILE} ${SOFTWARE_UPDATER_LOCALE_FILE}.2
		fgrep -v "SERVICE_PACK_CONFLICT" ${SOFTWARE_UPDATER_LOCALE_FILE}.2 > ${SOFTWARE_UPDATER_LOCALE_FILE}
		/bin/rm -f ${SOFTWARE_UPDATER_LOCALE_FILE}.2
	fi

	# Update the "appliance_local.properties" file
	if [ -f ${SOFTWARE_UPDATER_LOCALE_FILE} ]
	then
		cp ${SOFTWARE_UPDATER_LOCALE_FILE} ${LOCALE_FILE}
		rm -f ${SOFTWARE_UPDATER_LOCALE_FILE}
	fi

	logSoftwareUpdater "updateConflictProp(): Installation finished ($RC)"
	return $RC
}
		

softwareUpdater() {
	# Remove software updater log file if size > 2MB
	if [ -f $SOFTWARE_UPDATER_LOGFILE ]
	then
		fileSize=`du -sb $SOFTWARE_UPDATER_LOGFILE | awk '{ print $1 }'`
		if ((fileSize>2097152)); then
			/bin/rm -f $SOFTWARE_UPDATER_LOGFILE
		else
			logSoftwareUpdater ""
			logSoftwareUpdater ""
		fi
	fi

	logSoftwareUpdater "softwareUpdater: <$1>"

	if [ ! -f $1 ]
	then
		logSoftwareUpdater "softwareUpdater: $1 does not exist"
		echo "$1 does not exist"
		exit $ret
	fi
	
	# Retrieve the full path of the PACKAGE
	PACKAGE_NAME=`readlink -f $1`

	RPM_TYPE=`rpm --nosignature -q -p $PACKAGE_NAME --qf "%{Packager}\n"`
	if [ -z "$RPM_TYPE" ]
	then
		logSoftwareUpdater "softwareUpdater: invalid file type"
		echo "Error while installing $1"
		exit $ret
	fi

	# Verify the transport file format, dependencies and signature
	verifyTransport $PACKAGE_NAME "$RPM_TYPE"
	ret=$?
	if [ $ret != 0 ]
	then
                echo Error while installing $RPM_TYPE
		exit $ret
	fi
	
	# Retrieve the TRANSACTIONSETS and CONFLICT custom properties from the transport file
	SET_SP_CONFLICT=`getCustomProp $PACKAGE_NAME servicepackconflict`
	TRANSACTIONSETS=`getCustomProp $PACKAGE_NAME transactionsets`
	if [ -z "$TRANSACTIONSETS" ]
	then
		# If there is no transaction sets, use the default one according to the RPM TYPE
		if [ "$RPM_TYPE" = "$ADDON_TYPE" ]
		then
			TRANSACTIONSETS=".:u"
		else
			if [ "$RPM_TYPE" = "$HOTFIX_TYPE" ]
			then
				TRANSACTIONSETS=".:p"
			else
				if [ "$RPM_TYPE" = "$SERVICEPACK_TYPE" ]
				then
					TRANSACTIONSETS=".:f"
				else
					logSoftwareUpdater "softwareUpdater(): Unsupported RPM type: $RPM_TYPE"
			                echo Error while installing package
					exit 1
				fi
			fi
		fi
	fi
	
	# Create a temporary folder for the payload
	PAYLOAD_DIR=`mktemp -d`

	# Unpack the payload from the transport file into the temporary folder
	unpackTransport $PACKAGE_NAME $PAYLOAD_DIR
	ret=$?
	if [ $ret != 0 ]
	then
                echo Error while installing $RPM_TYPE
		exit $ret
	fi

	# Verify the payload installability, filter unneeded components
	verifyPayload $PAYLOAD_DIR $TRANSACTIONSETS
	ret=$?
	if [ $ret != 0 ]
	then
                echo Error while installing $RPM_TYPE
		exit $ret
	fi

	# Install the remaining payload components
	installPayload $PACKAGE_NAME $PAYLOAD_DIR $TRANSACTIONSETS "$RPM_TYPE"
	ret=$?
	if [ $ret != 0 ]
	then
                echo Error while installing $RPM_TYPE
		exit $ret
	fi

	logSoftwareUpdater "softwareUpdater(): Deleting payload folder"
	/bin/rm -rf $PAYLOAD_DIR

	# Update the SERVICEPACKCONFLICT local property if needed
	updateConflictProp $PACKAGE_NAME $SET_SP_CONFLICT
	ret=$?
	if [ $ret != 0 ]
	then
		echo Error while installing $RPM_TYPE
		exit $ret
	fi
	
	echo $RPM_TYPE successfully installed
}

softwareUpdater $1

	
