#!/bin/bash

###########################################################################################
#  (c) Copyright 2013 HID Global Corporation/ASSA ABLOY AB. All Rights Reserved.          #
#                                                                                         #
#  This program is an unpublished copyrighted work which is proprietary                   #
#  to HID Global Corporation/ASSA ABLOY AB. This computer program includes Confidential,  #
#  Proprietary Information and is a Trade Secret of HID Global Corporation/ASSA ABLOY AB. #
#  Any use, disclosure, modification and/or reproduction is prohibited                    #
#  unless authorized in writing by HID Global Corporation/ASSA ABLOY AB.                  #
#                                                                                         #
#  WARNING:  Unauthorized reproduction of this program as well as                         #
#  unauthorized preparation of derivative works based upon the                            #
#  program or distribution of copies by sale, rental, lease or                            #
#  lending are violations of federal copyright laws and state trade                       #
#  secret laws, punishable by civil and criminal penalties.                               #
###########################################################################################

UPDATER_HOME=/opt/hidglobal/ac.ac4linux
VERSION_DEB_NAME=unused # No version DEB for ActivClient
INSTALL_HISTORY_FILE=$UPDATER_HOME/installhistory.properties
LOCALE_FILE=$UPDATER_HOME/locale.properties
SOFTWARE_UPDATER_LOCALE_FILE=${LOCALE_FILE}_softwareupdater
SOFTWARE_UPDATER_LOGFILE=/var/log/activclient/softwareupdater.log
PAYLOAD_SUBFOLDER=Payload
FULLINSTALLER_TYPE="Full Installer"
ADDON_TYPE="Add-On"
HOTFIX_TYPE="Hot Fix"
SERVICEPACK_TYPE="Service Pack"
PATCH_TYPE="Patch"

logSoftwareUpdater()
{
	if [ -z "$1" ]
	then
		echo " " >> $SOFTWARE_UPDATER_LOGFILE
	else
		echo "`date`: $1" >> $SOFTWARE_UPDATER_LOGFILE
	fi
}

assert() {
	if [ "$1" -ne "0" ]; then
		logSoftwareUpdater "$2 ($1)"
		echo "$2 ($1)" >&2
		return $RC
	fi
}

getLocalProp() {
	logSoftwareUpdater "getLocalProp(): <$1>"
	fgrep "$1=" ${LOCALE_FILE} | cut -d"=" -f2 2> /dev/null
}

getCustomProp() {
	logSoftwareUpdater "getCustomProp(): <$1> <$2>"
	PACKAGE_NAME=$1
	dpkg-deb -f $PACKAGE_NAME $2 2> /dev/null | cut -d":" -f2- 2> /dev/null
}

setInstallHistoryProp() {
	logSoftwareUpdater "setInstallHistoryProp(): <$1> <$2> <$3> <$4>"
	PACKAGE_NAME=$1
	OP=$2
	PATCH_LIST=$3
	DEB_TYPE=$4
	[ -f $INSTALL_HISTORY_FILE ] || {
		touch $INSTALL_HISTORY_FILE
	}
	HOTFIX_NAME=`basename $PACKAGE_NAME`
	echo $PATCH_LIST | sed "s/ /\n/g" | while read PATCH_NAME
	do
		PATCH_NAME=`basename $PATCH_NAME`
		echo "$HOTFIX_NAME:$OP=$PATCH_NAME" >> $INSTALL_HISTORY_FILE
	done
	[ ".$DEB_TYPE" != "." ] && echo "$HOTFIX_NAME=$DEB_TYPE" >> $INSTALL_HISTORY_FILE
}

verifyTransport() {
	# Check if the dependencies are correctly installed
	logSoftwareUpdater "verifyTransport(): <$1> <$2>"
	PACKAGE_NAME=$1
	DEB_TYPE=$2

	dpkg --simulate -i $PACKAGE_NAME > tmp_deb_test.txt 2>&1
	ret=$?
	if [ $ret != 0 ]
	then
		logSoftwareUpdater "verifyTransport(): `cat tmp_deb_test.txt`"
	fi
	/bin/rm -f tmp_deb_test.txt
	
	# If this is a Service Pack, check if a conflict was registered
	if [ "$DEB_TYPE" = "$SERVICEPACK_TYPE" ]
	then
		NAME="SERVICE_PACK_CONFLICT"
		CONFLICTING=`getLocalProp $NAME`
		if [ -n "$CONFLICTING" ]
		then
			logSoftwareUpdater "verifyTransport(): Known conflict: $CONFLICTING"
			ret=1
		fi
	fi

	logSoftwareUpdater "verifyTransport(): return <$ret>"
	return $ret
}

unpackTransport() {
	logSoftwareUpdater "unpackTransport(): <$1> <$2> <$3>"
	TRANSPORT_FILE=$1
	PAYLOAD_BASE=$2
	PAYLOAD_DIR=$3
	[ -d $PAYLOAD_BASE ] || {
		logSoftwareUpdater "unpackTransport(): $PAYLOAD_BASE does not exist"
		return 1
	}
	[ -d $PAYLOAD_DIR ] || {
		logSoftwareUpdater "unpackTransport(): $PAYLOAD_DIR does not exist"
		return 1
	}
	
	dpkg-deb -x "$TRANSPORT_FILE" "$PAYLOAD_DIR"
	assert $? "failed to extract transport payload"
	return 0
}

filterPatches() {
	logSoftwareUpdater "filterPatches(): <$1>"
	DIR=$1
	for file in `ls $DIR/*.deb | fgrep -v $VERSION_DEB_NAME`
	do
		# Delete the patch if it is already installed
		PATCH_NAME=`dpkg-deb -f $file Package | cut -d":" -f2- 2> /dev/null`
		PATCH_VERSION=`dpkg-deb -f $file Version | cut -d":" -f2- 2> /dev/null`
		SETUP_NAME=`echo $PATCH_NAME | sed "s/-$PATCH_VERSION//g"`
		DEB_STATUS=`dpkg-query -l 2> /dev/null | fgrep $PATCH_NAME | fgrep $PATCH_VERSION | cut -d" " -f1 2> /dev/null`
		if [ "$DEB_STATUS" = "ii" ]
		then
			logSoftwareUpdater "filterPatches(): $SETUP_NAME-$PATCH_VERSION already installed"
			/bin/rm -f $file
		else
			# Delete the patch if the corresponding setup is not installed 
			SETUP_STATUS=`dpkg-query -l 2> /dev/null | fgrep $SETUP_NAME | cut -d" " -f1 2> /dev/null`
			[ !"$SETUP_STATUS" = "ii" ] && {
				logSoftwareUpdater "filterPatches(): $SETUP_NAME not installed, skip $SETUP_NAME-$PATCH_VERSION"
				/bin/rm -f $file
			}
		fi
	done
}

#   Returns:
#	 0: success
#	 1: failed
checkInstallability()
{
	logSoftwareUpdater "checkInstallability(): <$1> <$2>"
	PAYLOAD_DIR=$1
	TRANSACTIONSETS=$2
	RC=0

	# For all transaction sets, check if DEB can be installed
	echo $TRANSACTIONSETS | sed "s/,/\n/g" | sed "s/:/ /g" | while read TS OP
	do
		logSoftwareUpdater "checkInstallability(): Verify transaction set <$TS> <$OP>"
		DIR=$PAYLOAD_DIR/$TS

		case $OP in
			i) DEBOPT="-i" ;;
			u) DEBOPT="-?" ;; # TODO: ???
			f) DEBOPT="-?" ;; # TODO: ???
			p) DEBOPT="-i" ;;
		esac

		# Test install deb if any (if transactionset is '.', ignore version DEB)
		ls $PAYLOAD_DIR/$TS/*.deb 2> /dev/null | fgrep -v $VERSION_DEB_NAME 1> /dev/null
		if [ $? -eq 0 ]
		then
			dpkg --simulate $DEBOPT `ls $PAYLOAD_DIR/$TS/*.deb | fgrep -v $VERSION_DEB_NAME` > tmp_deb_test.txt 2>&1
			ret=$?
			if [ ! $ret -eq 0 ]
			then
				logSoftwareUpdater "checkInstallability(): `cat tmp_deb_test.txt`"
				/bin/rm -f tmp_deb_test.txt

				logSoftwareUpdater "checkInstallability(): Installation of <$TS> <$OP> failed"
				RC=1
			fi
		fi
	done

	# Test install of the version DEB (if any: only HF and SP have one)
	ls $PAYLOAD_DIR/*.deb 2> /dev/null | fgrep $VERSION_DEB_NAME 1> /dev/null
	[ $? -eq 0 ] && {
		dpkg --simulate -i `ls $PAYLOAD_DIR/*.deb | fgrep $VERSION_DEB_NAME` > tmp_deb_test.txt 2>&1
		ret=$?
		if [ ! $ret -eq 0 ]
		then
			logSoftwareUpdater "checkInstallability(): `cat tmp_deb_test.txt`"
			/bin/rm -f tmp_deb_test.txt

			logSoftwareUpdater "checkInstallability(): Installation of version DEB failed"
			RC=1
		fi
	}

	return $RC
}

#
# installThePayload
#   PACKAGE_NAME is the name of the transport DEB
#   TRANSACTIONSETS must not be empty:
#	 either it is the value set in the transport DEB
#	 or the default value in the form ".:<option>'
#	 where option is i,u,f,p depending on the package type
#   Returns:
#	 0: success
#	 1: failed
#
installThePayload() {
	logSoftwareUpdater "installThePayload(): <$1> <$2> <$3> <$4>"
	PACKAGE_NAME=$1
	PAYLOAD_DIR=$2
	TRANSACTIONSETS=$3
	DEB_TYPE=$4
	
	echo $TRANSACTIONSETS | sed "s/,/\n/g" | sed "s/:/ /g" | while read TS OP
	do
		logSoftwareUpdater "installThePayload(): Verify/Install transaction set $TS ($OP)"
		case $OP in
			i) DEBOPT="-i" ;;
			u) DEBOPT="-?" ;; # TODO: ???
			f) DEBOPT="-?" ;; # TODO: ???
			p) DEBOPT="-i" ;;
		esac
		# Install debs if any (if transactionset is '.', ignore version DEB)
		ls $PAYLOAD_DIR/$TS/*.deb 2> /dev/null | fgrep -v $VERSION_DEB_NAME 1> /dev/null
		if [ $? -eq 0 ]
		then
			dpkg $DEBOPT `ls $PAYLOAD_DIR/$TS/*.deb | fgrep -v $VERSION_DEB_NAME`
			if [ ! $? -eq 0 ]
			then
				logSoftwareUpdater "installThePayload(): Installation of $TS ($OP) failed"
				return 1
			fi

			# Update installhistory.properties with with operation, patch list and DEB type
			PATCH_LIST=`ls $PAYLOAD_DIR/$TS/*.deb | fgrep -v $VERSION_DEB_NAME`
			logSoftwareUpdater "installThePayload(): Register transaction set $PACKAGE_NAME=$PATCH_LIST"
			setInstallHistoryProp $PACKAGE_NAME $OP "$PATCH_LIST" "$DEB_TYPE"
		fi
		
	done
	RC=$? ; if [ ! $RC -eq 0 ] ; then return $RC ; fi

	# Install version DEB (if any: only HF and SP have one)
	ls $PAYLOAD_DIR/*.deb 2> /dev/null | fgrep $VERSION_DEB_NAME 1> /dev/null
	[ $? -eq 0 ] && {
		dpkg -i `ls $PAYLOAD_DIR/*.deb | fgrep $VERSION_DEB_NAME`
		if [ ! $? -eq 0 ]
		then
			logSoftwareUpdater "installThePayload(): Installation of version DEB failed"
			return 1
		fi
	}
	
	return 0
}

verifyPayload() {
	logSoftwareUpdater "verifyPayload(): <$1> <$2>"
	PAYLOAD_DIR=$1
	TRANSACTIONSETS=$2

	echo $TRANSACTIONSETS | sed "s/,/\n/g" | sed "s/:/ /g" | while read TS OP
	do
		logSoftwareUpdater "verifyPayload(): Verify transaction set <$TS> <$OP>"
		if [ "$OP" = "p" ]
		then
			filterPatches $PAYLOAD_DIR/$TS
		fi
	done

	checkInstallability $PAYLOAD_DIR $TRANSACTIONSETS
	if [ ! $? -eq 0 ]
	then
		logSoftwareUpdater "verifyPayload(): verify DEBs install failed"
		return 1
	fi

	return 0
}

installPayload() {
	logSoftwareUpdater "installPayload(): <$1> <$2> <$3> <$4>"

	PACKAGE_NAME=$1
	PAYLOAD_DIR=$2
	TRANSACTIONSETS=$3
	DEB_TYPE=$4

	installThePayload $PACKAGE_NAME $PAYLOAD_DIR $TRANSACTIONSETS "$DEB_TYPE"
	RC=$?
	if [ "$RC" -eq "0" ]
	then
		logSoftwareUpdater "installPayload(): Installation of $PACKAGE_NAME succeeded"
	fi
	logSoftwareUpdater "installPayload(): Installation finished ($RC)"
	return $RC
}

updateConflictProp() {
	logSoftwareUpdater "updateConflictProp(): <$1> <$2>"
	
	PACKAGE_NAME=$1
	SET_SP_CONFLICT=$2
	
	# Work on a temporary file
	touch $LOCALE_FILE
	cp $LOCALE_FILE ${SOFTWARE_UPDATER_LOCALE_FILE}
	
	# if the transport DEB has servicepackconflict set,
	# register it in appliance_local.properties
	if [ "$SET_SP_CONFLICT" = "true" ]
	then
		# look whether it is already defined
		NAME="SERVICE_PACK_CONFLICT"
		CONFLICTING=`getLocalProp $NAME`
		logSoftwareUpdater "updateConflictProp(): NAME=<$NAME> , CONFLICTING=<$CONFLICTING>"

		# update property file
		cp -p ${SOFTWARE_UPDATER_LOCALE_FILE} ${SOFTWARE_UPDATER_LOCALE_FILE}.2
		fgrep -v "$NAME" ${SOFTWARE_UPDATER_LOCALE_FILE}.2 > ${SOFTWARE_UPDATER_LOCALE_FILE}
		if [ -z "$CONFLICTING" ]
		then
			logSoftwareUpdater "updateConflictProp(): adding <$NAME=$PACKAGE_NAME> in file ${SOFTWARE_UPDATER_LOCALE_FILE}"
			echo "$NAME=$PACKAGE_NAME" >> ${SOFTWARE_UPDATER_LOCALE_FILE}
		else
			logSoftwareUpdater "updateConflictProp(): adding <$NAME=$PACKAGE_NAME,$CONFLICTING> in file ${SOFTWARE_UPDATER_LOCALE_FILE}"
			echo "$NAME=$PACKAGE_NAME,$CONFLICTING" >> ${SOFTWARE_UPDATER_LOCALE_FILE}
		fi
		/bin/rm -f ${SOFTWARE_UPDATER_LOCALE_FILE}.2
	fi
	if [ "$SET_SP_CONFLICT" = "false" ]
	then
		# remove the "SERVICE_PACK_CONFLICT" property
		cp -p ${SOFTWARE_UPDATER_LOCALE_FILE} ${SOFTWARE_UPDATER_LOCALE_FILE}.2
		fgrep -v "SERVICE_PACK_CONFLICT" ${SOFTWARE_UPDATER_LOCALE_FILE}.2 > ${SOFTWARE_UPDATER_LOCALE_FILE}
		/bin/rm -f ${SOFTWARE_UPDATER_LOCALE_FILE}.2
	fi

	# Update the "appliance_local.properties" file
	if [ -f ${SOFTWARE_UPDATER_LOCALE_FILE} ]
	then
		cp ${SOFTWARE_UPDATER_LOCALE_FILE} ${LOCALE_FILE}
		rm -f ${SOFTWARE_UPDATER_LOCALE_FILE}
	fi

	logSoftwareUpdater "updateConflictProp(): Installation finished ($RC)"
	return $RC
}
		

softwareUpdater() {
	# Remove software updater log file if size > 2MB
	if [ -f $SOFTWARE_UPDATER_LOGFILE ]
	then
		fileSize=`du -sb $SOFTWARE_UPDATER_LOGFILE | awk '{ print $1 }'`
		if ((fileSize>2097152)); then
			/bin/rm -f $SOFTWARE_UPDATER_LOGFILE
		else
			logSoftwareUpdater ""
			logSoftwareUpdater ""
		fi
	fi

	logSoftwareUpdater "softwareUpdater: <$1>"

	if [ ! -f $1 ]
	then
		logSoftwareUpdater "softwareUpdater: $1 does not exist"
		echo "$1 does not exist"
		exit $ret
	fi
	
	# Retrieve the full path of the PACKAGE
	PACKAGE_NAME=`readlink -f $1`

	DEB_TYPE=`getCustomProp $PACKAGE_NAME AC-Type`
	if [ -z "$DEB_TYPE" ]
	then
		logSoftwareUpdater "softwareUpdater: invalid file type"
		echo "Error while installing $1"
		exit $ret
	fi

	# Verify the transport file format, dependencies and signature
	verifyTransport $PACKAGE_NAME "$DEB_TYPE"
	ret=$?
	if [ $ret != 0 ]
	then
		echo Error while installing $DEB_TYPE
		exit $ret
	fi
	
	# Retrieve the TRANSACTIONSETS and CONFLICT custom properties from the transport file
	SET_SP_CONFLICT=`getCustomProp $PACKAGE_NAME AC-ServicePackConflict`
	TRANSACTIONSETS=`getCustomProp $PACKAGE_NAME AC-TransactionSets`
	if [ -z "$TRANSACTIONSETS" ]
	then
		# If there is no transaction sets, use the default one according to the DEB TYPE
		if [ "$DEB_TYPE" = "$ADDON_TYPE" ]
		then
			TRANSACTIONSETS=".:u"
		else
			if [ "$DEB_TYPE" = "$HOTFIX_TYPE" ]
			then
				TRANSACTIONSETS=".:p"
			else
				if [ "$DEB_TYPE" = "$SERVICEPACK_TYPE" ]
				then
					TRANSACTIONSETS=".:f"
				else
					logSoftwareUpdater "softwareUpdater(): Unsupported DEB type: $DEB_TYPE"
			                echo Error while installing package
					exit 1
				fi
			fi
		fi
	fi
	
	# Create a temporary folder for the payload
	PAYLOAD_BASE=`mktemp -d`
	PAYLOAD_DIR=$PAYLOAD_BASE/$PAYLOAD_SUBFOLDER
	mkdir $PAYLOAD_DIR

	# Unpack the payload from the transport file into the temporary folder
	unpackTransport $PACKAGE_NAME $PAYLOAD_BASE $PAYLOAD_DIR
	ret=$?
	if [ $ret != 0 ]
	then
                echo Error while installing $DEB_TYPE
		exit $ret
	fi

	# Verify the payload installability, filter unneeded components
	verifyPayload $PAYLOAD_DIR $TRANSACTIONSETS
	ret=$?
	if [ $ret != 0 ]
	then
                echo Error while installing $DEB_TYPE
		exit $ret
	fi

	# Install the remaining payload components
	installPayload $PACKAGE_NAME $PAYLOAD_DIR $TRANSACTIONSETS "$DEB_TYPE"
	ret=$?
	if [ $ret != 0 ]
	then
                echo Error while installing $DEB_TYPE
		exit $ret
	fi

	logSoftwareUpdater "softwareUpdater(): Deleting payload folder"
	/bin/rm -rf $PAYLOAD_BASE

	# Update the SERVICEPACKCONFLICT local property if needed
	updateConflictProp $PACKAGE_NAME $SET_SP_CONFLICT
	ret=$?
	if [ $ret != 0 ]
	then
                echo Error while installing $DEB_TYPE
		exit $ret
	fi
	
	echo $DEB_TYPE successfully installed
}

softwareUpdater $1

	
