#!/bin/sh

getRPMFirefoxSharedExtensionPath () {
FF_RPM_LIST=`rpm -qa | grep -Eio ^firefox`
if [ -z "$FF_RPM_LIST" ]
then
    #echo "Mozilla Firefox RPM not installed." >&2
    :
else
    for FF_RPM in $FF_RPM_LIST
    do
        FF_EXT_FOLDER_LIST=`rpm -ql $FF_RPM | grep -Ei 'share/mozilla/extensions/{ec8030f7-c20a-464f-9b0e-13a3a9e97384}$'`
        if [ -z "$FF_EXT_FOLDER_LIST" ]
        then
            #echo "Could not determine Mozilla Firefox RPM shared extensions folder" >&2
            :
        else
            for FF_EXT_FOLDER in $FF_EXT_FOLDER_LIST
            do
                echo $FF_EXT_FOLDER
                # install our extension in only one shared extension folder (even if several returned)
                break
            done
        fi
        # install our extension for only one Firefox RPM installation (even if several returned)
        break
    done
fi
}


getDEBFirefoxSharedExtensionPath () {
FF_DPKG_LIST=`dpkg -s firefox | grep -Eio "^Package:.*" | cut -d: -f2`
if [ -z "$FF_DPKG_LIST" ]
then
	# custom Firefox application for Debian OS
	FF_DPKG_LIST=`dpkg -s iceweasel | grep -Eio "^Package:.*" | cut -d: -f2`
fi
	if [ -z "$FF_DPKG_LIST" ]
	then
		#echo "Mozilla Firefox DPKG not installed." >&2
		:
	else
		for FF_DPKG in $FF_DPKG_LIST
		do
			FF_EXT_FOLDER_LIST=`dpkg -L $FF_DPKG | grep -Ei 'share/mozilla/extensions/{ec8030f7-c20a-464f-9b0e-13a3a9e97384}$'`
			
			if [ -z "$FF_EXT_FOLDER_LIST" ]
            		then
				# custom extension folder for Ubuntu OS
				FF_EXT_FOLDER_LIST=`dpkg -L $FF_DPKG | grep -Ei '/firefox/browser/extensions$'`
			fi

			if [ -z "$FF_EXT_FOLDER_LIST" ]
			then
				#echo "Could not determine Mozilla Firefox DPKG shared extensions folder" >&2
				:
			else
				for FF_EXT_FOLDER in $FF_EXT_FOLDER_LIST
				do
					echo $FF_EXT_FOLDER
					# install our extension in only one shared extension folder (even if several returned)
					break
				done
			fi

			# install our extension for only one Firefox DPKG installation (even if several returned)
			break
		done
	fi
}


if [ "$1" = "rpm" ]
then
    FirefoxSharedExtensionPath=$(getRPMFirefoxSharedExtensionPath)
elif [ "$1" = "deb" ]
then
    FirefoxSharedExtensionPath=$(getDEBFirefoxSharedExtensionPath)
fi

if [ ! -z "$FirefoxSharedExtensionPath" ]
then
    #determine PKCS#11 installation directory
    installdir_pkcs="/opt/hidglobal/ac.ac4linux.pkcs11/lib64/libac.pkcs220ong.so"
    if [ ! -f $installdir_pkcs ]
    then
        installdir_pkcs="/opt/hidglobal/ac.ac4linux.pkcs11/lib/libac.pkcs220ong.so"
        if [ ! -f $installdir_pkcs ]
        then
            #unrecoverable error, exit
            exit 1
        fi
    fi
    #configure our extension with correct PKCS#11 installation directory
    tmpdir_extension="/tmp/ac-ac4linux/mozillaextension/{00CC6330-A221-429B-9FAD-FD29EC560D7A}"
    subdir_js="chrome/content/overlay.js"
    tmpdir_js="$tmpdir_extension/$subdir_js"
    if [ ! -f $tmpdir_js ]
    then
        #unrecoverable error, exit
        exit 2
    fi
    sed -i 's|%p11path%|'$installdir_pkcs'|g' $tmpdir_js
    rc=$?
    if [ $rc -ne 0 ]
    then
        #unrecoverable error, exit
        exit $rc
    fi
    #install our extension in FirefoxSharedExtensionPath
    cp_err=$(cp -R $tmpdir_extension $FirefoxSharedExtensionPath 2>&1)
    if [ ! -z "$cp_err" ]
    then
        #echo "Failed to copy Mozilla Firefox shared extension with error: '" $cp_err "'"
        :
    else
        echo "Successfully installed Mozilla Firefox extension."
        installdir_extension="$FirefoxSharedExtensionPath/{00CC6330-A221-429B-9FAD-FD29EC560D7A}"
        #store FirefoxSharedExtensionPath for uninstallation script (product uninstallation cleanup)
        listfile="/opt/hidglobal/ac.ac4linux/setup/list"
        if [ ! -f $listfile ]
        then
        touch $listfile
        chmod 550 $listfile
        fi
        echo $installdir_extension >> $listfile
        #set appropriate permissions for extension
        find $installdir_extension -type d -exec chmod 775 {} +
        find $installdir_extension -type f -exec chmod 664 {} +
        chmod 775 $installdir_extension/$subdir_js
    fi
else
    #echo "Do not try to configure Fiferox extension (Firefox or Iceweasel not found)"
    :
fi

exit 0
