/* ---------------------------------------------------------------------------
   (c) Copyright 2014 HID Global Corporation/ASSA ABLOY AB. All Rights Reserved.

   This program is an unpublished copyrighted work which is proprietary
   to HID Global Corporation/ASSA ABLOY AB. This computer program includes Confidential,
   Proprietary Information and is a Trade Secret of HID Global Corporation/ASSA ABLOY AB.
   Any use, disclosure, modification and/or reproduction is prohibited
   unless authorized in writing by HID Global Corporation/ASSA ABLOY AB.

   WARNING:  Unauthorized reproduction of this program as well as
   unauthorized preparation of derivative works based upon the
   program or distribution of copies by sale, rental, lease or
   lending are violations of federal copyright laws and state trade
   secret laws, punishable by civil and criminal penalties.
--------------------------------------------------------------------------- */

const sACSecurityModule = "ActivClient PKCS #11";
const sAddOnID = "{00CC6330-A221-429B-9FAD-FD29EC560D7A}";
const nsIPKCS11 = Components.interfaces.nsIPKCS11;
const nsPKCS11ContractID = "@mozilla.org/security/pkcs11;1";
const MOZILLA_FIREFOX_ID = "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}";
const MOZILLA_THUNDERBIRD_ID = "{3550f703-e582-4d05-9a08-453d09bdfdc6}";
PKCS11_PUB_READABLE_CERT_FLAG = 0x1<<28;

window.addEventListener("load", function () { LoadSecurityModule(); }, false);

function LoadSecurityModule()
{
	try
	{
		// retrieve current Mozilla application infos
		var appInfo = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);
		// infoConsole("Running " + appInfo.name + " " + appInfo.version);
		
		if(appInfo.ID == MOZILLA_FIREFOX_ID)
		{
			var xulRuntime = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULRuntime);
			// alert("OS is " + xulRuntime.OS /*+ " " + xulRuntime.XPCOMABI*/);
			
			if(xulRuntime.OS == "WINNT")
			{
				var appVersion = Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator);
				if(appVersion.compare(appInfo.version, "4.0") >= 0)
				{
					Components.utils.import("resource://gre/modules/AddonManager.jsm"); 
					AddonManager.getAddonByID(sAddOnID, function(addon)
						{
							var sAddOnPath = addon.getResourceURI("").QueryInterface(Components.interfaces.nsIFileURL).file.path;
							AddSecurityModuleByFullPath(sAddOnPath + "/../../acpkcs211.dll");
						}
					);
				}
				else
				{
					var ext = Components.classes["@mozilla.org/extensions/manager;1"]
												.getService(Components.interfaces.nsIExtensionManager)
												.getInstallLocation(sAddOnID)
												.getItemLocation(sAddOnID);
					AddSecurityModuleByFullPath(ext.path + "/../../acpkcs211.dll");
				}
			}
			else if(xulRuntime.OS == "Linux")
			{
				var appVersion = Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator);
				if(appVersion.compare(appInfo.version, "3.5") >= 0)
				{
					AddSecurityModuleByFullPath("%p11path%");
				}
				else
				{
					// unmanaged application version
					return;
				}
			}
			else
			{
				var appVersion = Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator);
				if(appVersion.compare(appInfo.version, "3.5") >= 0)
				{
					AddSecurityModuleByFullPath("/Library/Frameworks/ac.ac4mac.pkcs11.framework/Versions/Current/Libraries/acpkcs220.dylib");
				}
				else
				{
					// unmanaged application version
					return;
				}
			}
		}
		else if(appInfo.ID == MOZILLA_THUNDERBIRD_ID)
		{
			var xulRuntime = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULRuntime);
			// infoConsole("OS is " + xulRuntime.OS /*+ " " + xulRuntime.XPCOMABI*/);
			
			if(xulRuntime.OS == "WINNT")
			{
				if(appVersion.compare(appInfo.version, "3.3") >= 0)
				{
					Components.utils.import("resource://gre/modules/AddonManager.jsm"); 

					AddonManager.getAddonByID(
						sAddOnID, function(addon)
						{
							var sAddOnPath = addon.getResourceURI("").QueryInterface(Components.interfaces.nsIFileURL).file.path;
							AddSecurityModuleByFullPath(sAddOnPath + "/../../acpkcs211.dll");
						}
					);
				}
				else
				{
					var ext = Components.classes["@mozilla.org/extensions/manager;1"]
													.getService(Components.interfaces.nsIExtensionManager)
													.getInstallLocation(sAddOnID)
													.getItemLocation(sAddOnID);
					AddSecurityModuleByFullPath(ext.path + "/../../acpkcs211.dll");
				}
			}
			else
			{
				// unmanaged OS
				return;
			}
		}
		else
		{
			// unmanaged Mozilla application
			return;
		}
	}
	catch (err)
	{
		// alert("err.message = " + err.message);
	}
}

function AddSecurityModuleByFullPath(fullPathP11)
{
	try
	{
		var PKCS11 = Components.classes[nsPKCS11ContractID].getService(nsIPKCS11);
		// alert("Will add 'Security Module' with path " + fullPathP11);
		PKCS11.addModule(sACSecurityModule, fullPathP11, PKCS11_PUB_READABLE_CERT_FLAG, 0);
	}
	catch (err)
	{
		// alert("err.message = " + err.message);
	}
}

function infoConsole(Message)
{
    var consoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
    consoleService.logStringMessage("ActivClient extension: " + Message);
	// alert(Message);
 }
